from __future__ import annotations

from logbook.base import (
    CRITICAL as CRITICAL,
    DEBUG as DEBUG,
    ERROR as ERROR,
    INFO as INFO,
    NOTICE as NOTICE,
    NOTSET as NOTSET,
    TRACE as TRACE,
    WARNING as WARNING,
    Logger as Logger,
    LoggerGroup as LoggerGroup,
    LogLevel as LogLevel,
    LogRecord as LogRecord,
    Processor as Processor,
)
